--[[
Title: LibFreeVoice
Author: Joequincy of Korialstrasz (US Horde)
Major Version: 1.0
Minor Version: $Revision: 00001 $
Date: $Date: 2007-01-23 17:42:17 -0800 (Tue, 23 Dec 2007) $
]]

local MAJOR_VERSION = "FreeVoice-1.0"
local MINOR_VERSION = 10000 + tonumber(("$Revision: 1 $"):match("(%d+)"))

-- This ensures the code is only executed if the libary doesn't already exist, or is a newer version
if not AceLibrary then error(MAJOR_VERSION .. " requires AceLibrary.") end
if not AceLibrary:IsNewVersion(MAJOR_VERSION, MINOR_VERSION) then return end

if not AceLibrary:HasInstance("AceOO-2.0") then error(MAJOR_VERSION .. " requires AceOO-2.0.") end

local AceEvent = AceLibrary:HasInstance("AceEvent-2.0") and AceLibrary("AceEvent-2.0")
local Tablet = AceLibrary:HasInstance("Tablet-2.0") and AceLibrary("Tablet-2.0")
local Dewdrop = AceLibrary:HasInstance("Dewdrop-2.0") and AceLibrary("Dewdrop-2.0")
local AceAddon

local epsilon = 1e-5
local _G = getfenv(0)

local SHOW_ICON = "Show icon"
local SHOW_ICON_DESC = "Show the plugins icon on the panel."
local SHOW_TEXT = "Show text"
local SHOW_TEXT_DESC = "Show the plugins text on the panel."
local SHOW_COLORED_TEXT = "Show colored text"
local SHOW_COLORED_TEXT_DESC = "Allow the plugin to color its text."
local DETACH_TOOLTIP = "Detach tooltip"
local DETACH_TOOLTIP_DESC = "Detach the tooltip from the panel."
local LOCK_TOOLTIP = "Lock tooltip"
local LOCK_TOOLTIP_DESC = "Lock the tooltips position. When the tooltip is locked, you must use Alt to access it with your mouse."
local POSITION = "Position"
local POSITION_DESC = "Position the plugin on the panel."
local POSITION_LEFT = "Left"
local POSITION_RIGHT = "Right"
local POSITION_CENTER = "Center"
local ATTACH_TO_MINIMAP = "Attach to minimap"
local ATTACH_TO_MINIMAP_DESC = "Attach the plugin to the minimap instead of the panel."
local HIDE_FUBAR_PLUGIN = "Hide plugin"
local HIDE_FUBAR_PLUGIN_CMD = "Hidden"
local HIDE_FUBAR_PLUGIN_DESC = "Hide the plugin from the panel or minimap, leaving the addon running."
local OTHER = "Other"
local CLOSE = "Close"
local CLOSE_DESC = "Close the menu."

local AceOO = AceLibrary("AceOO-2.0")
local FreeVoice = AceOO.Mixin {
	"GetTitle",
	"GetName",
	"GetCategory",
	"SetFontSize",
	"GetFrame",
	"Show",
	"Hide",
	"GetPanel",
	"IsTextColored",
	"ToggleTextColored",
	"IsMinimapAttached",
	"ToggleMinimapAttached",
	"Update",
	"UpdateDisplay",
	"UpdateData",
	"UpdateText",
	"UpdateTooltip",
	"SetIcon",
	"GetIcon",
	"CheckWidth",
	"SetText",
	"GetText",
	"IsIconShown",
	"ToggleIconShown",
	"ShowIcon",
	"HideIcon",
	"IsTextShown",
	"ToggleTextShown",
	"ShowText",
	"HideText",
	"IsTooltipDetached",
	"ToggleTooltipDetached",
	"DetachTooltip",
	"ReattachTooltip",
	"GetDefaultPosition",
	"SetPanel",
	"IsLoadOnDemand",
	"IsDisabled",
	"CreateBasicPluginFrame",
	"CreatePluginChildFrame",
	"OpenMenu",
	"AddImpliedMenuOptions",
}
local MinimapContainer


function FreeVoice:UpdateDisplay()
	self:UpdateText()
	self:UpdateTooltip()
end

function FreeVoice:UpdateText()
	if type(self.OnTextUpdate) == "function" then
		if not self:IsDisabled() then
			self:OnTextUpdate()
		end
	elseif self:IsTextShown() then
		self:SetText(self:GetTitle())
	end
end

function FreeVoice:IsDisabled()
	return self.IsActive and not self:IsActive() or false
end

function FreeVoice:GetTitle()
	local name = self.title or self.name
	if not name then
		FuBarPlugin:error("You must provide self.title or self.name")
	end
	local title = select(3, name:find("FuBar %- (.-)%s*$"))
	if not title then
		title = name
	end
	return title:gsub("|c%x%x%x%x%x%x%x%x", ""):gsub("|r", "")
end

local good = nil
local function CheckFuBar()
	if not good then
		good = FuBar and tonumber(FuBar.version:sub(1, 3)) and tonumber(FuBar.version:sub(1, 3)) >= 2 and true
	end
	return good
end

function FreeVoice:IsMinimapAttached()
	if not CheckFuBar() then
		return true
	end
	return self.panel == MinimapContainer
end



function FreeVoice:UpdateTooltip()
	if self.blizzardTooltip then
		if GameTooltip:IsOwned(self:IsMinimapAttached() and self.minimapFrame or self.frame) then
			GameTooltip:Hide()
			
			local frame = self:IsMinimapAttached() and self.minimapFrame or self.frame
			local anchor
			if frame:GetTop() > GetScreenHeight() / 2 then
				local x = frame:GetCenter()
				if x < GetScreenWidth() / 2 then
					anchor = "ANCHOR_BOTTOMRIGHT"
				else
					anchor = "ANCHOR_BOTTOMLEFT"
				end
			else
				local x = frame:GetCenter()
				if x < GetScreenWidth() / 2 then
					anchor = "ANCHOR_TOPLEFT"
				else
					anchor = "ANCHOR_TOPRIGHT"
				end
			end
			GameTooltip:SetOwner(frame, anchor)
			if type(self.OnTooltipUpdate) == "function" and not self:IsDisabled() then
				self:OnTooltipUpdate()
			end
			GameTooltip:Show()
		end
		return
	elseif self.overrideTooltip then
		if type(self.OnTooltipUpdate) == "function" and not self:IsDisabled() then
			self:OnTooltipUpdate()
		end
		return
	end
	if not Tablet then return end

	FuBarPlugin.RegisterTablet(self)
	if self:IsMinimapAttached() and not self:IsTooltipDetached() and self.minimapFrame then
		Tablet:Refresh(self.minimapFrame)
	else
		Tablet:Refresh(self.frame)
	end
end

